@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: Yelper.cpp,v 1.5 2006/06/12 04:39:59 bmsmith Exp $
+ *  $Id: Yelper.cpp,v 1.6 2006/07/23 18:50:01 dscorgie Exp $
  */
 
 #include <mozilla-config.h>
@@ -224,7 +224,9 @@ Yelper::Print (YelpPrintInfo *print_info, PRBool preview, int *prev_pages)
   rv = print->GetGlobalPrintSettings (getter_AddRefs (settings));
   NS_ENSURE_SUCCESS (rv, rv);
 
-  PrintListener::SetPrintSettings (print_info, settings);
+  rv = PrintListener::SetPrintSettings (print_info, preview, settings);
+
+  NS_ENSURE_SUCCESS (rv, rv);
 
   nsCOMPtr<PrintListener> listener = new PrintListener (print_info, print);
 
