@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: gtkentryaction.c,v 1.3 2006/01/15 17:17:28 dscorgie Exp $
+ *  $Id: gtkentryaction.c,v 1.5 2006/05/29 18:44:49 dscorgie Exp $
  */
 
 #include "config.h"
@@ -119,7 +119,7 @@ sync_label (GtkAction *gaction,
   GtkEntryAction *action = GTK_ENTRY_ACTION (gaction);
   GtkToolItem *item = GTK_TOOL_ITEM (proxy);
   GtkLabel *label = GTK_LABEL (g_object_get_data (G_OBJECT (item), "label"));
-  char *text;
+  char *text = NULL;
 
   g_object_get (action, "label", &text, NULL);
   gtk_label_set_text_with_mnemonic (label, text);
@@ -127,6 +127,7 @@ sync_label (GtkAction *gaction,
     gtk_widget_show (GTK_WIDGET (label));
   else
     gtk_widget_hide (GTK_WIDGET (label));
+  g_free (text);
 }
 
 static void
@@ -260,7 +261,6 @@ gtk_entry_action_finalize (GObject *object)
   GtkEntryActionPrivate *priv = action->priv;
 
   g_free (priv->text);
-  g_free (priv);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
