@@ -30,13 +30,12 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <bonobo/bonobo-main.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnome/libgnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
 #include <string.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-url.h>
 
 #include "yelp-bookmarks.h"
 #include "yelp-db-pager.h"
@@ -458,7 +457,7 @@ static const GtkActionEntry entries[] = {
       "F1", 
       N_("Help On this application"),
       G_CALLBACK (window_help_contents_cb) },
-    { "About", GNOME_STOCK_ABOUT,
+    { "About", GTK_STOCK_ABOUT,
       N_("_About"),
       NULL,
       NULL,
@@ -596,7 +595,7 @@ history_push_back (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoBack");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
-    title = gtk_window_get_title (window);
+    title = (gchar *) gtk_window_get_title (GTK_WINDOW (window));
 
     if (g_str_equal (title, "Loading..."))
 	entry->page_title = g_strdup ("Unknown Page");
@@ -614,7 +613,8 @@ history_push_back (YelpWindow *window)
 		      entry);
 		      
 
-    gtk_menu_shell_prepend (window->priv->back_menu, entry->menu_entry);
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+			    entry->menu_entry);
     gtk_widget_show (entry->menu_entry);
 }
 
@@ -642,7 +642,7 @@ history_push_forward (YelpWindow *window)
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
 
-    title = gtk_window_get_title (window);
+    title = (gchar *) gtk_window_get_title (GTK_WINDOW (window));
 
     if (g_str_equal (title, "Loading..."))
 	entry->page_title = g_strdup ("Unknown Page");
@@ -659,7 +659,8 @@ history_push_forward (YelpWindow *window)
 		      entry);
 		      
 
-    gtk_menu_shell_prepend (window->priv->forward_menu, entry->menu_entry);
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+			    entry->menu_entry);
     gtk_widget_show (entry->menu_entry);
 }
 
@@ -799,7 +800,8 @@ history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 
     while (!g_str_equal (latest->page_title, entry->page_title)) {
 	priv->history_forward = g_slist_prepend (priv->history_forward, latest);
-	gtk_container_remove (priv->back_menu, latest->menu_entry);
+	gtk_container_remove (GTK_CONTAINER (priv->back_menu), 
+			      latest->menu_entry);
 	
 	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
 	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
@@ -808,7 +810,8 @@ history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 					     latest);
 	
 	
-	gtk_menu_shell_prepend (window->priv->forward_menu, latest->menu_entry);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+				latest->menu_entry);
 	gtk_widget_show (latest->menu_entry);
 
 	
@@ -847,7 +850,8 @@ history_forward_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 
     while (!g_str_equal (latest->page_title, entry->page_title)) {
 	priv->history_back = g_slist_prepend (priv->history_back, latest);
-	gtk_container_remove (priv->forward_menu, latest->menu_entry);
+	gtk_container_remove (GTK_CONTAINER (priv->forward_menu), 
+			      latest->menu_entry);
 	
 	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
 	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
@@ -855,7 +859,8 @@ history_forward_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 					     G_CALLBACK (history_back_to),
 					     latest);
 
-	gtk_menu_shell_prepend (window->priv->back_menu, latest->menu_entry);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+				latest->menu_entry);
 	gtk_widget_show (latest->menu_entry);
 
 	
@@ -1260,13 +1265,13 @@ window_populate (YelpWindow *window)
 
     toolbar = gtk_ui_manager_get_widget(priv->ui_manager, "ui/tools");
 
-    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), f_proxy, 0);
-    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), b_proxy, 0);
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (f_proxy), 0);
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (b_proxy), 0);
 
     /*To keep the seperator but get rid of the extra widget*/
     toolitem = gtk_ui_manager_get_widget(priv->ui_manager, "ui/tools/GoFor");
 
-    gtk_container_remove (toolbar, toolitem);
+    gtk_container_remove (GTK_CONTAINER (toolbar), toolitem);
 
     priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/main_popup");
     priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
@@ -1881,7 +1886,7 @@ static void
 window_disconnect (YelpWindow *window)
 {
     YelpWindowPriv *priv;
-    YelpPager      *pager;
+    YelpPager      *pager = NULL;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -2877,32 +2882,12 @@ window_copy_mail_cb (GtkAction *action, YelpWindow *window)
 static void
 window_help_contents_cb (GtkAction *action, YelpWindow *window)
 {
-    GnomeProgram *program = gnome_program_get ();
-    GError *error = NULL;
-
-    g_return_if_fail (YELP_IS_WINDOW (window));
-
-    gnome_help_display_desktop (program, "user-guide", 
-                                "user-guide", "yelp", &error);
-
-    if (error) {
-	GtkWidget *dialog;
-
-	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
- 	                                 0,
-	                                 GTK_MESSAGE_ERROR,
-	                                 GTK_BUTTONS_CLOSE,
-	                                 _("Could not display help for Yelp.\n"
-	                                 "%s"),
-	                                 error->message);
-
-	g_signal_connect_swapped (dialog, "response",
-	                          G_CALLBACK (gtk_widget_destroy),
-	                          dialog);
-	gtk_widget_show (dialog);
-
-	g_error_free (error);
-    }
+    /* Since we are already running, and are calling ourselves,
+     * we can make this function easy by just creating a new window
+     * and avoiding calling gnome_help_display_desktop
+     */
+    g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0, 
+		   "ghelp:user-guide#yelp");
 }
 
 static void
